import logging
from os import path, symlink, unlink
import numpy as np
import time
from arthur.imaging import  calculate_lag
from arthur.plot import plot_image, plot_lag, plot_chan_power, plot_corr_mat, plot_diff
from arthur.constants import NUM_CHAN
from matplotlib import pyplot as plt

filename_template = "S{band}_R01-63_{timestamp}_{figure}.png"

logger = logging.getLogger(__name__)


def write_images_to_disk(date, img_data, corr_data, lags, prev_data,
                         chan_data, chan_row, frequency, prefix):
    """
    Calculate and write various images to disk.

    Note that this function has side affects to keep track of history.

    args:
        date (datetime.datetime)
        img_data (numpy.array): the image data
        corr_data (numpy.array): the correlation data
        lags (numpy.array): history of difference between header timestamp and
                            arrival
        prev_data (numpy.array): the previous image, used for calculate diff
        chan_data (numpy.array): the historical chan data, will be
                                 updated using chan_row
        chan_row (numpy.array): the new chan array
        frequency (float): the frequency of the observation
        prefix (str): where to write the images to
    """
    lags += [calculate_lag(date).seconds]
    if prev_data is None:
        prev_data = img_data

        # update historical data
    chan_data = np.roll(chan_data, 1)
    chan_data[:, 0] = chan_row
    diff_data = img_data - prev_data
    prev_data = img_data

    figures = (
        ('image', plot_image(date, img_data, frequency)),
        ('lag', plot_lag(lags)),
        ('chan', plot_chan_power(chan_data)),
        ('corr', plot_corr_mat(corr_data, frequency, date)),
        ('diff', plot_diff(diff_data, frequency, date)),
    )

    timestamp = date.strftime("T%d-%m-%Y-%H-%M-%S%Z")
    some_constant = 195312.5  # not sure what this is, ask Folkert
    for name, figure in figures:
        args = {'band': int(frequency / some_constant),
                'timestamp': timestamp,
                'figure': name}
        filename = filename_template.format(**args)
        full_filename = path.join(prefix, filename)
        logger.info('writing {}'.format(filename))
        figure.savefig(full_filename)  # pad_inches=0, bbox_inches='tight')
        plt.close(figure)  # required to free up memory

        # symlink to latest version
        link_target = path.join(prefix, name + '.png')
        if path.islink(link_target):
            unlink(link_target)
        symlink(filename, link_target)


def make_imaging_closure(prefix, frequency):
    """
    iterate over iterable containing visibilities, makes images and writes them
    into prefix:

    args:
        iterable (iterable): an iterable of generators generating visibilites
        prefix (str): a filesystem prefix where to write the images to
        frequency (float): the central frequency

    return:

    """
    # initialise historical structures
    lags = []
    prev_data = None
    chan_data = np.zeros((NUM_CHAN, 60), dtype=np.float32)

    # we create a closure here so we can store state (
    def closure(date, img_data, corr_data, chan_row):
        """
        args:
            date:
            img_data:
            corr_data:
            chan_row:

        """
        nonlocal prev_data
        write_images_to_disk(date, img_data, corr_data, lags, prev_data,
                             chan_data, chan_row, frequency, prefix)
        prev_data = img_data
    return closure



#!/usr/bin/env python3

import sys
import logging
from arthur.stream import loop_images_in_path, setup_stream_pipe, stream

if len(sys.argv) < 2:
    print("This will stream a folder of casacore images to youtube")
    print()
    print("usage: {} <images> <youtube secret>".format(sys.argv[0]))
    sys.exit(1)
else:
    path = sys.argv[1]
    secret = sys.argv[2]


logging.basicConfig(level=logging.DEBUG)
images = loop_images_in_path(path)
pipe = setup_stream_pipe("rtmp://a.rtmp.youtube.com/live2/" + secret)
stream(images, pipe)



import os
from casacore.images import image as casa_image
from PIL import Image
import subprocess
import time
import monotonic
import logging
import atexit

logger = logging.getLogger(__name__)

FPS = 25

cmd = ["ffmpeg",
       # for ffmpeg always first set input then output

       # silent audio
       '-f', 'lavfi',
       '-i', 'anullsrc=channel_layout=stereo:sample_rate=44100',

       # image
       '-re',
       '-f', 'rawvideo',           # probably required for reading from stdin
       '-s', '1024x1024',          # should match image size
       '-pix_fmt', 'gray',
       '-i', '-',                  # read from stdin

       # encoding settings
       "-r", str(FPS),             # the framerate
       "-vcodec", "libx264",       # probably required for flv & rtmp
       "-preset", "ultrafast",     # the encoding quality preset
       "-g", "20",
       "-codec:a", "libmp3lame",   # mp3 for audio
       "-ar", "44100",             # 44k audio rate
       "-threads", "6",
       "-bufsize", "512k",
       "-f", "flv",                # required for rtmp
       ]


def setup_stream_pipe(rtmp_url):
    """
    Setup a encoding process where you can pipe images to.

    args:
        rtmp_url (str): a rtmp url, for example rtmp://a.rtmp.youtube.com/live2/{SECRET}

    returns:
        subprocess.Popen: a subprocess pipe. Use pipe.stdin.write for writing images.
    """
    pipe = subprocess.Popen(cmd + [rtmp_url], stdin=subprocess.PIPE)
    atexit.register(pipe.kill)
    return pipe


def serialize_array(array):
    """
    serialize a numpy array into a binary stream which can be streamed

    args:
        array (numpy.array)

    returns:
        bytes
    """
    data = array.squeeze()
    data -= data.min()
    max_ = data.max()
    if max_ > 0:
        data *= (255 / data.max())
    arr = data.astype('uint8')
    im = Image.fromarray(arr, 'L')
    return im.tobytes()


def loop_images_in_path(path):
    """
    args:
        path (str): path to folder containing images

    returns:
        generator: yields casacore images
    """
    images = sorted([os.path.join(path, i) for i in os.listdir(path)])
    while True:
        for image_path in images:
            yield casa_image(image_path).getdata()


def stream(frame, pipe):
    """
    stream the images returned by generated to rtmp server.

    args:
        frame: an image frame
        pipe (subprocess.Popen): a pipe created with setup_stream_pipe()
    """
    logger.debug("streaming new image")
    serialised = serialize_array(frame)
    for i in range(FPS):
        pipe.stdin.write(serialised)
    duty_cycle = 1  # seconds

    if pipe.poll():
        print("looks like the video encoder died!")
        return
    time.sleep(duty_cycle - monotonic.monotonic() % duty_cycle)



import arthur
import unittest
from os import path

import arthur.imaging
import arthur.plot
import arthur.io

HERE = path.dirname(__file__)
VIS_PATH = path.join(HERE, 'data/2aartfaac.vis')
FRQ = 58398437.5  # Central observation frequency in Hz


class TestArthur(unittest.TestCase):
    def test_arthur(self):
        start_time, body = arthur.io.read_data(open(VIS_PATH, 'rb'))
        image, corr_data, chan_row = arthur.imaging.full_calculation(body, FRQ)
        arthur.plot.plot_image(start_time, image, FRQ)



import logging
from arthur.writer import make_imaging_closure
from arthur.imaging import full_calculation
from arthur.stream import setup_stream_pipe, stream
from concurrent.futures import ProcessPoolExecutor
from multiprocessing import Manager


logger = logging.getLogger(__name__)


def image_queue_pusher(date, body, frequency, queue):
    """
    Do calculations and put results in a queue. Run this in a thread or
    multiprocess.
    """
    result = full_calculation(body, frequency)
    queue.put([date] + list(result))


def queue_repeater(in_queue, out_queues):
    """
    Listen for incoming objects on a queue, replay on out queues.

    args:
        in_queue (Queue): in coming
        out_queues (list):
    """
    logger.debug("repeater starting")
    while True:
        result = in_queue.get()
        logger.debug("Going to repeat something on repeat queue ({})".format(in_queue.qsize()))
        for queue in out_queues:
            queue.put(result)
        logger.debug("Done repeating")


def write_scheduler(queue, frequency, media_root):
    """
    Queue listener that will make images and write them to disk. Run in thread
    or multiprocess.
    """
    imager_writer = make_imaging_closure(media_root, frequency)
    while True:
        result = queue.get()
        logger.debug("recieved on writer queue ({})".format(queue.qsize()))
        imager_writer(*result)
        logger.debug("done writing")


def stream_scheduler(queue, youtube_url):
    """
    Queue listener that will stream images to youtube. Run in thread
    or multiprocess.
    """
    pipe = setup_stream_pipe(youtube_url)
    while True:
        _, img_data, _, _ = queue.get()
        logger.debug("Got something from stream queue ({})".format(queue.qsize()))
        stream(img_data, pipe)
        logger.debug("done streaming")


def big_fat_loop_that_does_everything(generator, frequency,
                                      media_root, youtube_url):
        manager = Manager()
        repeat_queue = manager.Queue()
        writer_queue = manager.Queue()
        stream_queue = manager.Queue()

        with ProcessPoolExecutor() as executor:
            executor.submit(queue_repeater, repeat_queue,
                            [writer_queue, stream_queue])
            executor.submit(write_scheduler, writer_queue,
                            frequency, media_root)
            executor.submit(stream_scheduler, stream_queue, youtube_url)
            for date, body in generator:
                logging.info("processing image timestamped {}".format(date))
                executor.submit(image_queue_pusher, date, body,
                                frequency, repeat_queue)
            print("done! now what")




#!/usr/bin/env python3

import sys
import numpy as np
from arthur.imaging import full_calculation, calculate_lag
from arthur.io import read_full
from arthur.plot import plot_image, plot_lag, plot_chan_power, plot_corr_mat, plot_diff
from arthur.constants import NUM_CHAN
from matplotlib import pyplot

FRQ = 58398437.5  # Central observation frequency in Hz

def main():
    if len(sys.argv) < 2:
        print("Image the first set of visibilites from a visibilities file")
        print()
        print("usage: {} <file>".format(sys.argv[0]))
        sys.exit(1)
    else:
        path = sys.argv[1]

    # define them here so we can access them out of for loop scope
    lags = []
    prev_data = date = img_data = corr_data = diff_data = None
    chan_data = np.zeros((NUM_CHAN, 60), dtype=np.float32)

    for date, body in read_full(path):
        img_data, corr_data, chan_row = full_calculation(body, FRQ)
        lags += [calculate_lag(date).seconds]
        if prev_data is None:
            prev_data = img_data
        chan_data = np.roll(chan_data, 1)
        chan_data[:, 0] = chan_row
        diff_data = img_data - prev_data
        prev_data = img_data

    fig_img = plot_image(date, img_data, FRQ)
    fig_lag = plot_lag(lags)
    fig_chan = plot_chan_power(chan_data)
    fig_cm = plot_corr_mat(corr_data, FRQ, date)
    fig_diff = plot_diff(diff_data, FRQ, date)
    pyplot.show()

if __name__ == '__main__':
    main()



import unittest
from arthur import imaging
from arthur import constants
import numpy as np


FRQ = 58398437.5  # Central observation frequency in Hz


class testImaging(unittest.TestCase):
    def tets_correlation_matrix(self):
        data = np.zeros((constants.NUM_ANTS, constants.NUM_ANTS), dtype=np.complex64)
        antennas = constants.NUM_ANTS
        imaging.correlation_matrix(data, antennas)

    def test_calc_channels(self):
        data = np.zeros((constants.NUM_ANTS, constants.NUM_ANTS),
                        dtype=np.complex64)
        imaging.calc_channels(data)

    def test_make_image(self):
        cm = np.zeros((constants.NUM_ANTS, constants.NUM_ANTS),
                     dtype=np.complex64)
        imaging.make_image(cm, FRQ)



import ephem
import numpy as np
from arthur import constants
from matplotlib import pyplot as plt


def calculate_annotations(date):
    """
    Calculates the positions of a list of bodies in the sky, as seen from
    CS002 on LOFAR and based on the datetime.

    args:
        date (datetime.datetime): For what moment to calculate positions

    returns:
        dict: name of object name, value a tuple for position.
    """
    obs = ephem.Observer()
    obs.pressure = 0  # To prevent refraction corrections.
    obs.lon, obs.lat = '6.869837540', '52.915122495'  # CS002 on LOFAR
    obs.date = date
    objects = [
        ephem.Moon(),
        ephem.Jupiter(),
        ephem.Sun(),
        ephem.readdb('Cas-A,f|J,23:23:26.0,58:48:00,99.00,2000'),
        ephem.readdb('Cyg-A,f|J,19:59:28.35,40:44:02,99.00,2000'),
        ephem.readdb('NCP,f|J, 0,90:00:00,99.00,2000'),
        ephem.readdb('Galactic Center,f|J, 17:45:40.0,-29:00:28.1,99.00,2000'),
    ]

    annotations = {}
    for o in objects:
        o.compute(obs)
        if o.alt > 0:
            l = -(np.cos(o.alt) * np.sin(o.az))
            m = (np.cos(o.alt) * np.cos(o.az))
            annotations[o.name] = (l, m)

    return annotations


def plot_image(startdatetime, img_data, frequency):
    """
    Plot a sky image with object overlay.

    args:
        date (datetime.time): start datetime of observation
        img_data (numpy.array): a numpy array with image data

    returns:
        matplotlib.figure.Figure
    """
    annotations = calculate_annotations(startdatetime)

    # fft image
    img_min = img_data.min()
    img_max = img_data.max()
    l = np.linspace(-1, 1, constants.IMAGE_RES)
    m = np.linspace(-1, 1, constants.IMAGE_RES)
    mask = np.ones((constants.IMAGE_RES, constants.IMAGE_RES))
    xv, yv = np.meshgrid(l, m)
    mask[np.sqrt(np.array(xv ** 2 + yv ** 2)) > 1] = np.NaN

    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1)
    img = ax.imshow(img_data * mask, interpolation='bilinear',
               cmap=plt.get_cmap('jet'), vmin=img_min, vmax=img_max,
               extent=[l[0], l[-1], m[0], m[-1]])
    fig.colorbar(img, cmap=plt.get_cmap('jet'), norm=plt.Normalize(vmin=img_min,
                                                              vmax=img_max))

    arrowprops = {'facecolor': 'white', 'width': 1, 'headwidth': 4,
                  'headlength': 5, 'shrink': 0.15, 'edgecolor': 'white'}
    for source, pos in annotations.items():
        ax.annotate(source, xy=pos, xytext=(pos[0] + 0.1, pos[1] + 0.1),
                     color='white', arrowprops=arrowprops,
                     horizontalalignment='left', verticalalignment='bottom')
    moment = startdatetime.strftime("%Y-%m-%d_%H:%M:%S %Z")
    ax.set_title('XX-%.2f MHz-%s' % (frequency / 1e6, moment))
    ax.set_xlabel(r'$\leftarrow East - West \rightarrow$')
    ax.set_ylabel(r'$\leftarrow South - North \rightarrow$')
    return fig


def plot_lag(lag_data):
    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1)
    ax.plot(lag_data)
    ax.set_title('Lag')
    ax.set_xlabel('Time (s)')
    ax.set_ylabel('Lag (s)')
    return fig


def plot_chan_power(chan_data):
    # Check if the chan_data has non-zero values. If the stations are turned
    # off, correlator still runs, but generates visibilities with value 0. The
    # dataselection below then generates a ValueError.
    if (np.any(chan_data)):
        chan_min, chan_max = chan_data[chan_data != 0].min(), chan_data[
            chan_data != 0].max()
    else:
        chan_min = 0
        chan_max = 0

    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1)
    img = ax.imshow(chan_data, interpolation='nearest',
                    cmap=plt.get_cmap('afmhot'), vmin=chan_min, vmax=chan_max)
    fig.colorbar(img, cmap=plt.get_cmap('afmhot'),
                 norm=plt.Normalize(vmin=chan_min, vmax=chan_max))
    ax.set_title('Channel Power (dB)')
    ax.set_xlabel('Time (s)')
    ax.set_ylabel('Channels')
    return fig


def plot_corr_mat(corr_data, freq, date):
    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1)
    img = ax.imshow(corr_data, interpolation='nearest',
               cmap=plt.get_cmap('jet'), vmin=corr_data.min(),
               vmax=corr_data.max(), extent=[288, 0, 288, 0])
    ax.set_title('Dipole Covariance XX-%.2f MHz-%s' % (
    freq / 1e6, date.strftime("%Y-%m-%d_%H:%M:%S %Z")))
    ax.set_xticks([])
    ax.set_yticks([])
    return fig


def plot_diff(diff_data, freq, date):
    annotations = calculate_annotations(date)
    fig = plt.figure()
    ax = fig.add_subplot(1, 1, 1)
    diff_min, diff_max = diff_data.min(), diff_data.max()
    img = ax.imshow(diff_data, interpolation='bilinear',
                    cmap=plt.get_cmap('coolwarm'), vmin=diff_min, vmax=diff_max)
    fig.colorbar(img, cmap=plt.get_cmap('coolwarm'),
                 norm=plt.Normalize(vmin=diff_min, vmax=diff_max))

    arrowprops = {'facecolor': 'white', 'width': 1, 'headwidth': 4,
                  'headlength': 5, 'shrink': 0.15, 'edgecolor': 'white'}

    for source, pos in annotations.items():
        ax.annotate(source, xy=pos, xytext=(pos[0] + 0.1, pos[1] + 0.1),
                     color='white', arrowprops=arrowprops,
                     horizontalalignment='left', verticalalignment='bottom')
    moment = date.strftime("%Y-%m-%d_%H:%M:%S %Z")
    plt.title('XX-%.2f MHz Difference-%s' % (freq/1e6, moment))
    plt.xlabel(r'$\leftarrow East - West \rightarrow$')
    plt.ylabel(r'$\leftarrow South - North \rightarrow$')
    plt.xticks([])
    plt.yticks([])
    return fig



import struct
from datetime import datetime
import numpy as np
from arthur import constants
import socket
import logging
from six import BytesIO

try:
    from functools import lru_cache
except ImportError:
    from backports.functools_lru_cache import lru_cache

logger = logging.getLogger(__name__)


def parse_header(raw_header):
    """
    parse a AARTFAAC visibilities triangle header.

    args:
        header (bytes): an AARTFAAC header

    returns:
        tuple: start date (datetime.time), body (numpy.array)

    """
    magic, _, start, end = struct.unpack("<IIdd", raw_header[0:24])
    assert magic == constants.HDR_MAGIC
    return start, end


def parse_body(raw_body):
    """
    parse a raw set of bytes and return a reshaped numpy array

    args:
        raw_body (bytes): the raw matrix

    returns:
        numpy.array
    """
    serial_body = np.fromstring(raw_body,  dtype=np.complex64)
    indices = create_indices(0, constants.NUM_CHAN, constants.NUM_BSLN,
                             constants.NUM_POLS)
    body = reshape_body(serial_body, indices, constants.NUM_CHAN,
                        constants.NUM_BSLN)
    return body


def reader(producer, bytes_):
    """
    Read an amount of bytes from a socket or file

    args:
        producer: a socket or file object
        bytes_ (int): number of bytes to read

    returns:
        bytes: the read data
    """
    if type(producer) == socket.socket:
        result = BytesIO()
        count = bytes_
        while count > 0:
            recv = producer.recv(count)
            if len(recv) == 0:
                logger.warning("client closed connection")
                producer.close()
                raise IOError("client closed connection")

            count -= len(recv)
            result.write(recv)
        return result.getvalue()
    else:
        # assuming file like interface
        data = producer.read(bytes_)
        if len(data) != bytes_:
            logger.warning("end of file")
            raise IOError("end of file")
        return data


def read_data(handler):
    """
    read one data window from a file or socket like object.

    args:
        handler: an object with a file like interface (read)

    returns:
        tuple (date, body)
    """
    raw_header = reader(handler, constants.LEN_HDR)
    start_time, end_time = parse_header(raw_header)
    raw_body = reader(handler, constants.LEN_BDY)
    body = parse_body(raw_body)
    date = datetime.utcfromtimestamp(start_time)
    return date, body


def read_full(path):
    """
    Open file in path and read until end.

    args:
        path (str): a path to a visibilities file
    returns:
        iterator
    """
    handler = open(path, 'rb')
    while True:
        try:
            yield read_data(handler)
        except IOError:
            raise StopIteration


@lru_cache()
def create_indices(pol, channels, baselines, pols):
    """
    Creates the indices to obtain an upper triangle of visibilities for a
    certain polarization from a data block as constructed via the GPU
    correlator.

    args:
        pol (int): probably polarisation
        channels (int): number of channels
        baselines (int): number of baselines
        pols (int): number of polarisations
    """
    I = np.empty((channels, baselines), dtype=np.uint32)
    for b in range(baselines):
        for c in range(channels):
            I[c, b] = pol + c * pols + b * pols * channels
    return I.reshape(I.size, 1)


def reshape_body(raw_body, indices, channels, baselines):
    """
    Restructure the data into CHANNELS x BASELINES matrix for a certain
    polarization.

    args:
        raw_body (numpy.array): a flat numpy array
        channels (int): number of channels
        baselines (int): number of baselines

    return:
        numpy.array
    """
    result = np.zeros((channels, baselines), dtype=np.complex64)
    np.copyto(result.reshape(result.size, 1), np.take(raw_body, indices))
    return result


def listen_socket(port=5000, host='localhost'):
    """
    Listen on socket, wait for a client to connect. Then the client is
    expected to stream raw visilibities. If the client disconnects the
    server will wait for a reconnect.

    args:
        port (int): which port to listen on
        host (str): which interface to listen on

    returns:
        iterator

    """
    sid = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sid.bind((host, port))
    sid.listen(1)

    while True:
        logger.info("waiting for connection on port {}...".format(port))
        handler, address = sid.accept()
        logger.info("connection from {}:{}".format(*address))
        while True:
            try:
                yield read_data(handler)
            except IOError:
                break




import time
from datetime import datetime
import numpy as np
from arthur import constants
from arthur.data import load_antpos
from arthur.gridding import grid


def correlation_matrix(data, antennas):
    """
    Construct the mean correlation matrix from the CHANNELS x BASELINES data
    matrix.

    args:
        antennas (int): number of antennas

    return:
        numpy.array
    """
    tril_index = np.tril_indices(constants.NUM_ANTS)
    triangle = data.mean(axis=0)
    cm = np.zeros((antennas, antennas), dtype=np.complex64)
    cm[tril_index] = np.conjugate(triangle)
    cm = cm.transpose()
    cm[tril_index] = triangle
    return cm


def calc_channels(data):
    """
    Construct the mean normalized absolute value per channel in dB scale
    """
    pol = np.abs(data.mean(axis=1))
    # prevent div by zero
    if np.sum(pol) > 0:
        pol = 10.0 * np.log10(pol)
    return pol


def make_image(cm, frequency):
    """
    Create an image from the correlation matrix
    """
    # No calibration vector applied
    gains = np.ones((1, constants.NUM_ANTS), dtype=np.complex64)
    cal_mat = gains.conj().transpose() * gains

    # Apply supplied calibration vector
    cm = cal_mat * cm

    U, V = load_antpos(constants.ANTPOS)
    mDuv = constants.C_MS / frequency / 2.0
    gridvis = grid(U, V, cm, mDuv, constants.IMAGE_RES)
    gridvis = np.fft.fftshift(gridvis)
    gridvis = np.flipud(np.fliplr(gridvis))
    gridvis = np.conjugate(gridvis)
    return np.real(np.fft.fftshift(np.fft.fft2(gridvis)))


def historical_channels(body, new_row):
    """
    Make a matrix of historical channel data

    TODO: add history

    args:
        body (numpy.array): a sky image
    returns:
        numpy.array: a matrix containing historical channel data

    """
    chan_data = np.zeros((constants.NUM_CHAN, 60), dtype=np.float32)
    chan_data = np.roll(chan_data, 1)
    chan_data[:, 0] = calc_channels(body)
    return chan_data


def calculate_lag(date):
    return datetime.now() - date


def historical_lag(start_time):
    # lag
    lag_data = np.zeros(60, dtype=np.float32)
    lag_data = np.roll(lag_data, 1)
    lag_data[0] = (time.time() - start_time) - 1.0


def full_calculation(body, frequency):
    # correlation matrix
    cm = correlation_matrix(body, constants.NUM_ANTS)
    corr_data = np.abs(cm)
    corr_data[np.diag_indices(constants.NUM_ANTS)] = np.min(corr_data)

    image = make_image(cm, frequency)
    chan_row = calc_channels(body)

    return image, corr_data, chan_row


try:
    from functools import lru_cache
except ImportError:
    from backports.functools_lru_cache import lru_cache

from arthur import constants
import numpy as np


@lru_cache()
def load_antpos(path):
    """
    Load antenna positions and compute U,V coordinates

    args:
        path (str): path to antenna pos file

    returns:
        tuple: (numpy.array, numpy.array)
    """
    A = np.loadtxt(path)
    R = np.array([[-0.1195950000, -0.7919540000, 0.5987530000],
                  [0.9928230000, -0.0954190000, 0.0720990000],
                  [0.0000330000, 0.6030780000, 0.7976820000]])
    L = A.dot(R)

    U = np.zeros((constants.NUM_ANTS, constants.NUM_ANTS), dtype=np.float64)
    V = np.zeros((constants.NUM_ANTS, constants.NUM_ANTS), dtype=np.float64)

    for a1 in range(constants.NUM_ANTS):
        for a2 in range(constants.NUM_ANTS):
            U[a1, a2] = L[a1, 0] - L[a2, 0]
            V[a1, a2] = L[a1, 1] - L[a2, 1]
    return U, V


from os import path

IMAGE_RES = 256
C_MS = 299792458.0
NUM_ANTS = 288
NUM_BSLN = int((NUM_ANTS ** 2 + NUM_ANTS) / 2)
NUM_CHAN = 63
NUM_POLS = 2
LEN_HDR = 512
LEN_BDY = NUM_BSLN * NUM_POLS * NUM_CHAN * 8  # complex64 (8 bytes)

HDR_MAGIC = 0x3B98F002
HERE = path.dirname(__file__)
ANTPOS = path.join(HERE, 'lba_outer.dat')



import unittest

from arthur.data import load_antpos
import numpy as np
from arthur import gridding
from arthur import constants

FRQ = 58398437.5  # Central observation frequency in Hz


class testGridding(unittest.TestCase):
    def test_load_antpos(self):
        load_antpos(constants.ANTPOS)

    def test_grid(self):
        U = np.zeros((constants.NUM_ANTS, constants.NUM_ANTS), dtype=np.float64)
        V = np.zeros((constants.NUM_ANTS, constants.NUM_ANTS), dtype=np.float64)
        C = np.zeros((constants.NUM_ANTS, constants.NUM_ANTS), dtype=np.complex64)
        duv = constants.C_MS / FRQ / 2.0
        size = constants.IMAGE_RES
        gridding.grid(U, V, C, duv, size)





from setuptools import setup, find_packages

__version__ = "0.5"


install_requires = (
    'pillow',
    'python-casacore',
    'astropy',
    'ephem',
    'matplotlib',
    'scipy',
    'monotonic',
    'backports.functools_lru_cache',
    'six',
)


scripts = [
    'scripts/arthur-plot.py',
    'scripts/arthur-stream.py',
]

setup(
    name="aartfaac-arthur",
    version=__version__,
    packages=find_packages(),
    scripts=scripts,
    install_requires=install_requires,
    package_data={
        '': ['*.txt', '*.rst'],
        'arthur': ['lba_outer.dat'],
    },
    author="Gijs Molenaar",
    author_email="gijs@pythonic.nl",
    description="AARTFAAC visualisation utilities",
    license="GPL3",
    keywords="science astronomy aartfaac",
    url="https://github.com/transientskp/aartfaac-arthur",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 3.5",
        "Topic :: Scientific/Engineering",
        ],
    #ext_modules=[Extension("gridding_fast",
    #                       sources=["arthur/gridding_fast.pyx"])],
)



import numpy as np
from arthur import constants


def grid(U, V, C, duv, size):
    G = np.zeros((size, size), np.complex64)
    G.fill(0)
    for a1 in range(constants.NUM_ANTS):
        for a2 in range(constants.NUM_ANTS):
            p = 1.0
            if a1 == a2:
                p = 0.5

            u = U[a1, a2] / duv + size / 2 - 1
            v = V[a1, a2] / duv + size / 2 - 1

            w = int(np.floor(u))
            e = int(np.ceil(u))
            s = int(np.floor(v))
            n = int(np.ceil(v))

            west_power = p - (u - w)
            east_power = p - (e - u)
            south_power = p - (v - s)
            north_power = p - (n - v)

            south_west_power = south_power * west_power
            north_west_power = north_power * west_power
            south_east_power = south_power * east_power
            north_east_power = north_power * east_power

            G[s, w] += south_west_power * C[a1, a2]
            G[n, w] += north_west_power * C[a1, a2]
            G[s, e] += south_east_power * C[a1, a2]
            G[n, e] += north_east_power * C[a1, a2]
    return G
